from __future__ import annotations

from PyQt5 import QtCore, QtGui, QtWidgets

from gui_qt.lib.delegates import ColorListEditor


class ObjectWidget(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.layout = QtWidgets.QGridLayout()
        self.layout.setContentsMargins(3, 3, 3, 3)

        self.color_label = QtWidgets.QLabel('Color')
        self.layout.addWidget(self.color_label, 0, 0, 1, 1)
        self.color_box = ColorListEditor(self)
        self.layout.addWidget(self.color_box, 0, 1, 1, 2)
        self.setLayout(self.layout)

    def collect_args(self) -> dict:
        return {'color': self.color_box.currentData(QtCore.Qt.UserRole)}

    @staticmethod
    def parse_point(x_widget: QtWidgets.QLineEdit, y_widget: QtWidgets.QLineEdit) -> None | tuple[float, float]:
        x = x_widget.text()
        if not x:
            return
        y = y_widget.text()
        if not y:
            return

        try:
            return float(x), float(y)
        except ValueError:
            return


class LineWidget(ObjectWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.pos_label = QtWidgets.QLabel('Position')
        self.layout.addWidget(self.pos_label, 1, 0, 1, 1)
        self.pos_lineedit = QtWidgets.QLineEdit()
        self.pos_lineedit.setValidator(QtGui.QDoubleValidator())
        self.layout.addWidget(self.pos_lineedit, 1, 1, 1, 2)

        self.orientation_label = QtWidgets.QLabel('Orientation')
        self.layout.addWidget(self.orientation_label, 2, 0, 1, 1)
        self.orient_combobox = QtWidgets.QComboBox()
        self.orient_combobox.addItems(['Horizontal', 'Vertical'])
        self.layout.addWidget(self.orient_combobox, 2, 1, 1, 2)
        self.layout.setRowStretch(3, 1)

    def collect_args(self):
        dic = super().collect_args()
        pos = self.pos_lineedit.text()
        if not pos:
            return

        try:
            dic['pos'] = float(pos)
        except ValueError:
            return

        dic['angle'] = self.orient_combobox.currentIndex() * 90

        return dic


class MultiPointWidget(ObjectWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.loop_checkbox = QtWidgets.QCheckBox('Close loop')
        self.layout.addWidget(self.loop_checkbox, 1, 0, 1, 3)

        self.table_widget = QtWidgets.QTableWidget()
        self.table_widget.setColumnCount(2)
        self.table_widget.setHorizontalHeaderLabels(['x', 'y'])
        header = self.table_widget.horizontalHeader()
        header.setStretchLastSection(True)
        self.layout.addWidget(self.table_widget, 2, 0, 1, 3)

        self.addButton = QtWidgets.QPushButton('Add point')
        self.layout.addWidget(self.addButton, 3, 1, 1, 1)
        self.layout.setRowStretch(3, 1)
        self.addButton.clicked.connect(self.new_point)

        self.removeButton = QtWidgets.QPushButton('Remove point')
        self.layout.addWidget(self.removeButton, 3, 2, 1, 1)
        self.layout.setRowStretch(3, 1)
        self.removeButton.clicked.connect(self.less_point)

    def new_point(self, _):
        row = self.table_widget.rowCount()
        self.table_widget.setRowCount(row+1)
        placeholder = ['x', 'y']
        for column in range(2):
            line_edit = QtWidgets.QLineEdit()
            line_edit.setFrame(False)
            line_edit.setPlaceholderText(placeholder[column])
            line_edit.setValidator(QtGui.QDoubleValidator())

            self.table_widget.setCellWidget(row, column, line_edit)

    def less_point(self, _):
        self.table_widget.removeRow(self.table_widget.rowCount()-1)

    def collect_args(self):
        dic = super().collect_args()

        pts = []
        if self.table_widget.rowCount() <= 1:
            return

        for row in range(self.table_widget.rowCount()):
            next_pt = self.parse_point(self.table_widget.cellWidget(row, 0), self.table_widget.cellWidget(row, 1))
            if next_pt is None:
                return
            pts.append(next_pt)

        if pts:
            dic['pts'] = pts
            dic['closed'] = self.loop_checkbox.isChecked()
            return dic
        else:
            return


class RectangleWidget(ObjectWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.leftbottom_label = QtWidgets.QLabel('Lower left')
        self.layout.addWidget(self.leftbottom_label, 1, 0, 1, 1)
        self.left_x = QtWidgets.QLineEdit()
        self.left_x.setPlaceholderText('x')
        self.layout.addWidget(self.left_x, 1, 1, 1, 1)
        self.left_y = QtWidgets.QLineEdit()
        self.left_y.setPlaceholderText('y')
        self.left_x.setValidator(QtGui.QDoubleValidator())
        self.left_y.setValidator(QtGui.QDoubleValidator())
        self.layout.addWidget(self.left_y, 1, 2, 1, 1)

        self.righttop_label = QtWidgets.QLabel('Upper right')
        self.layout.addWidget(self.righttop_label, 2, 0, 1, 1)
        self.right_x = QtWidgets.QLineEdit()
        self.right_x.setPlaceholderText('x')
        self.layout.addWidget(self.right_x, 2, 1, 1, 1)
        self.right_y = QtWidgets.QLineEdit()
        self.right_y.setPlaceholderText('y')
        self.right_x.setValidator(QtGui.QDoubleValidator())
        self.right_y.setValidator(QtGui.QDoubleValidator())
        self.layout.addWidget(self.right_y, 2, 2, 1, 1)

        self.layout.setRowStretch(3, 1)

    def collect_args(self):
        dic = super().collect_args()
        left = self.parse_point(self.left_x, self.left_y)
        if left is None:
            return
        dic['left'] = left

        right = self.parse_point(self.right_x, self.right_y)
        if right is None:
            return
        dic['right'] = right

        return dic


class EllipseWidget(ObjectWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.centre_label = QtWidgets.QLabel('Centre')
        self.layout.addWidget(self.centre_label, 1, 0, 1, 1)
        self.centre_x = QtWidgets.QLineEdit()
        self.centre_x.setPlaceholderText('x')
        self.layout.addWidget(self.centre_x, 1, 1, 1, 1)
        self.centre_y = QtWidgets.QLineEdit()
        self.centre_y.setPlaceholderText('y')
        self.centre_x.setValidator(QtGui.QDoubleValidator())
        self.centre_y.setValidator(QtGui.QDoubleValidator())
        self.layout.addWidget(self.centre_y, 1, 2, 1, 1)

        self.axes_label = QtWidgets.QLabel('Axes')
        self.layout.addWidget(self.axes_label, 2, 0, 1, 1)
        self.width = QtWidgets.QLineEdit()
        self.width.setPlaceholderText('width')
        self.layout.addWidget(self.width, 2, 1, 1, 1)
        self.height = QtWidgets.QLineEdit()
        self.height.setPlaceholderText('Height')
        self.width.setValidator(QtGui.QDoubleValidator())
        self.width.setValidator(QtGui.QDoubleValidator())
        self.layout.addWidget(self.height, 2, 2, 1, 1)

        self.layout.setRowStretch(3, 1)

    def collect_args(self):
        dic = super().collect_args()
        centre = self.parse_point(self.centre_x, self.centre_y)
        if centre is None:
            return
        dic['centre'] = centre

        axes = self.parse_point(self.width, self.height)
        if axes is None:
            return
        dic['axes'] = axes

        return dic


class TextWidget(ObjectWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.centre_label = QtWidgets.QLabel('Centre')
        self.layout.addWidget(self.centre_label, 1, 0, 1, 1)
        self.centre_x = QtWidgets.QLineEdit()
        self.centre_x.setPlaceholderText('x')
        self.layout.addWidget(self.centre_x, 1, 1, 1, 1)
        self.centre_y = QtWidgets.QLineEdit()
        self.centre_y.setPlaceholderText('y')
        self.centre_x.setValidator(QtGui.QDoubleValidator())
        self.centre_y.setValidator(QtGui.QDoubleValidator())
        self.layout.addWidget(self.centre_y, 1, 2, 1, 1)

        self.text_label = QtWidgets.QLabel('Text')
        self.layout.addWidget(self.text_label, 2, 0, 1, 1)
        self.text_lineedit =QtWidgets.QLineEdit()
        self.layout.addWidget(self.text_lineedit, 2, 1, 1, 2)

        self.layout.setRowStretch(3, 1)

    def collect_args(self):
        dic = super().collect_args()
        centre = self.parse_point(self.centre_x, self.centre_y)
        if centre is None:
            return
        dic['centre'] = centre

        dic['text'] = self.text_lineedit.text()

        return dic
